// Vertex class
// derived from: cls_vertex.h
// created 10/2/99, by mark.carlson@ieee.org

#include "vertex.h"

Vertex::Vertex() {}
Vertex::Vertex(const Vertex& inV) { x = inV.x; y = inV.y; z = inV.z; }	
Vertex::Vertex(float inX, float inY, float inZ) { x = inX; y = inY; z = inZ; }

Vertex& Vertex::operator = (const Vertex& inV) { x = inV.x; y = inV.y; z = inV.z; return(*this); }
Vertex& Vertex::operator = (const double inF) { x = (float)inF; y = (float)inF; z = (float)inF; return(*this); }
Vertex& Vertex::operator += (const Vertex& inV) { x += inV.x; y += inV.y; z += inV.z; return(*this); }
Vertex& Vertex::operator -= (const Vertex& inV) { x -= inV.x; y -= inV.y; z -= inV.z; return(*this); }
Vertex& Vertex::operator *= (const Vertex& inV) { x *= inV.x; y *= inV.y; z *= inV.z; return(*this); }
Vertex& Vertex::operator /= (const Vertex& inV) { x /= inV.x; y /= inV.y; z /= inV.z; return(*this); }
Vertex& Vertex::operator *= (float inScale) { x *= inScale; y *= inScale; z *= inScale; return(*this); }
Vertex& Vertex::operator /= (float inScale) { x /= inScale; y /= inScale; z /= inScale; return(*this); }

float Vertex::Length() const { return((float)sqrt(x*x + y*y + z*z)); }
float Vertex::Normalize() { float a(Length()); float b(1.0f/a); x *= b; y *= b; z *= b; return(a); }
float Vertex::XPlusY() const { return((float)x + y); }

Vertex operator - (const Vertex& inV) { return(Vertex(-inV.x, -inV.y, -inV.z)); }
Vertex operator + (const Vertex& inV1, const Vertex& inV2) { return(Vertex(inV1.x+inV2.x, inV1.y+inV2.y, inV1.z+inV2.z)); }
Vertex operator - (const Vertex& inV1, const Vertex& inV2) { return(Vertex(inV1.x-inV2.x, inV1.y-inV2.y, inV1.z-inV2.z)); }
Vertex operator * (const Vertex& inV1, const Vertex& inV2) { return(Vertex(inV1.x*inV2.x, inV1.y*inV2.y, inV1.z*inV2.z)); }
Vertex operator / (const Vertex& inV1, const Vertex& inV2) { return(Vertex(inV1.x/inV2.x, inV1.y/inV2.y, inV1.z/inV2.z)); }
Vertex operator * (const Vertex& inV, float inScale) { return(Vertex(inV.x*inScale, inV.y*inScale, inV.z*inScale)); }
Vertex operator / (const Vertex& inV, float inScale) { return(Vertex(inV.x/inScale, inV.y/inScale, inV.z/inScale)); }
float operator & (const Vertex& inV1, const Vertex& inV2) { return((inV2 - inV1).Length()); }
float operator | (const Vertex& inV1, const Vertex& inV2) { return(inV1.x*inV2.x + inV1.y*inV2.y + inV1.z*inV2.z); }
Vertex operator ^ (const Vertex& inV1, const Vertex& inV2){
  return(Vertex(inV1.y*inV2.z-inV1.z*inV2.y, inV1.z*inV2.x-inV1.x*inV2.z, inV1.x*inV2.y-inV1.y*inV2.x));
}

