// Vertex class
// derived from: cls_vertex.h
// created 10/2/99, by mark.carlson@ieee.org

#ifndef INCLUDED_CLS_VERTEX
#define INCLUDED_CLS_VERTEX

#ifndef INCLUDED_MATH
#define INCLUDED_MATH
#include <math.h>
#endif

class Vertex{
 public:
	float x, y, z;

	Vertex(); // default constructor, no initialization
	Vertex(const Vertex& inV); // copy constructor
	Vertex(float inX, float inY, float inZ); // component constructor

	Vertex& operator = (const Vertex& inV); // assignment
	Vertex& operator = (const double inF); // assign initial value
	Vertex& operator += (const Vertex& inV); // component add assignment
	Vertex& operator -= (const Vertex& inV); // component sub assignment
	Vertex& operator *= (const Vertex& inV); // component mul assignment
	Vertex& operator /= (const Vertex& inV); // component div assignment
	Vertex& operator *= (float inScale); // scalar mul assignment
	Vertex& operator /= (float inScale); // scalar div assignment

	float Length() const; // length of vector (distance of point from origin)
	float Normalize(); // normalize vector, return length
	float XPlusY() const;// sends back the sum of x and y;

	friend Vertex operator - (const Vertex& inV); // unary component negate
	friend Vertex operator + (const Vertex& inV1, const Vertex& inV2); // component add
	friend Vertex operator - (const Vertex& inV1, const Vertex& inV2); // component sub
	friend Vertex operator * (const Vertex& inV1, const Vertex& inV2); // component mul
	friend Vertex operator / (const Vertex& inV1, const Vertex& inV2); // component div
	friend Vertex operator * (const Vertex& inV, float inScale); // scalar mul
	friend Vertex operator / (const Vertex& inV, float inScale); // scalar div
	friend float operator & (const Vertex& inV1, const Vertex& inV2); // distance between points
	friend float operator | (const Vertex& inV1, const Vertex& inV2); // dot product
	friend Vertex operator ^ (const Vertex& inV1, const Vertex& inV2); // cross product
};


#endif // INCLUDED_CLS_VERTEX
